<?php
/**
 * Gmaper for Elementor
 * Google maps for Elementor editor
 * Exclusively on https://1.envato.market/gmaper-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.4
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Cherviakov Vlad (vladchervjakov@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\GmaperElementor;

use Merkulove\GmaperElementor\Unity\Plugin;
use Merkulove\GmaperElementor\Unity\Settings;
use Merkulove\GmaperElementor\Unity\UI;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Caster class contain main plugin logic.
 *
 * @since 1.0.0
 *
 **/
final class Caster {

	/**
	 * The one true Caster.
	 *
     * @since 1.0.0
     * @access private
	 * @var Caster
	 **/
	private static $instance;

    /**
     * Setup the plugin.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function setup() {

        /** Define hooks that runs on both the front-end as well as the dashboard. */
        $this->both_hooks();

        /** Define public hooks. */
        $this->public_hooks();

        /** Define admin hooks. */
        $this->admin_hooks();

    }

    /**
     * Define hooks that runs on both the front-end as well as the dashboard.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function both_hooks() {

        /** Hooks which should work everywhere in backend and frontend. */

    }

    /**
     * Register all of the hooks related to the public-facing functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function public_hooks() {

        /** Work only on frontend area. */
        if ( is_admin() ) { return; }

        /** Public hooks and filters. */

    }

    /**
     * Register all of the hooks related to the admin area functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function admin_hooks() {

        /** Work only in admin area. */
        if ( ! is_admin() ) { return; }

        /** Admin hooks and filters here. */
        $options = Settings::get_instance()->options;
        if ( !$options['google_maps_api_key'] ) {
            add_action( 'admin_footer', [ $this, 'key_notice' ] );
        }

    }

	/**
	 * This method used in register_activation_hook
	 * Everything written here will be done during plugin activation.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function activation_hook() {

		/** Activation hook */

	}

    /**
     * Show admin warning, if we need API Key.
     *
     * @since 1.0.0
     * @access public
     **/
    public function key_notice() {

        /** Get current screen. */
        $screen = get_current_screen();
        if ( null === $screen ) { return; }

        /** Gmaper Settings Page. */
        if ( in_array( $screen->base , Plugin::get_menu_bases() )  ) {

            /** Render "Before you start" message. */
            UI::get_instance()->render_snackbar(
                esc_html__( 'This plugin uses the Google Maps API Key. Set up your Google Cloud Platform project before the start.', 'gmaper-elementor' ),
                'warning', // Type
                -1, // Timeout
                true, // Is Closable
                [ [ 'caption' => 'Get Key', 'link' => 'https://docs.merkulov.design/how-to-get-google-map-api-key/' ] ] // Buttons
            );

        } else { ?>

            <div class="settings-error notice notice-warning">
                <p><strong><?php esc_html_e( 'Gmaper: Before you begin', 'gmaper-elementor' ); ?></strong></p>
                <p><?php esc_html_e( 'This plugin uses Google Maps API. You need to set up your Google Cloud Platform project and Google Maps API key. Visit', 'gmaper-elementor' ); ?>
                    <a href="https://docs.merkulov.design/how-to-get-google-map-api-key/"
                       target="_blank"><?php esc_html_e( 'Online Documentation', 'gmaper-elementor' ); ?></a> <?php esc_html_e( 'for more details.', 'gmaper-elementor' ); ?>
                </p>
            </div>

            <?php

        }

    }

	/**
	 * Main Caster Instance.
	 * Insures that only one instance of Caster exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return Caster
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
